@@ -8,19 +8,21 @@ const Main = imports.ui.main;
 const PanelMenu = imports.ui.panelMenu;
 const PopupMenu = imports.ui.popupMenu;
 
-const Gettext = imports.gettext.domain('gnome-shell-extensions');
-const _ = Gettext.gettext;
-const N_ = x => x;
-
 const Me = ExtensionUtils.getCurrentExtension();
 const PlaceDisplay = Me.imports.placeDisplay;
 
+const _ = ExtensionUtils.gettext;
+const N_ = x => x;
+
 const PLACE_ICON_SIZE = 16;
 
-var PlaceMenuItem = GObject.registerClass(
 class PlaceMenuItem extends PopupMenu.PopupBaseMenuItem {
-    _init(info) {
-        super._init({
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor(info) {
+        super({
             style_class: 'place-menu-item',
         });
         this._info = info;
@@ -74,7 +76,7 @@ class PlaceMenuItem extends PopupMenu.PopupBaseMenuItem {
         this._icon.gicon = info.icon;
         this._label.text = info.name;
     }
-});
+}
 
 const SECTIONS = [
     'special',
@@ -83,10 +85,13 @@ const SECTIONS = [
     'network',
 ];
 
-let PlacesMenu = GObject.registerClass(
 class PlacesMenu extends PanelMenu.Button {
-    _init() {
-        super._init(0.0, _('Places'));
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor() {
+        super(0.0, _('Places'));
 
         let label = new St.Label({
             text: _('Places'),
@@ -131,14 +136,16 @@ class PlacesMenu extends PanelMenu.Button {
 
         this._sections[id].actor.visible = places.length > 0;
     }
-});
+}
 
+/** */
 function init() {
     ExtensionUtils.initTranslations();
 }
 
 let _indicator;
 
+/** */
 function enable() {
     _indicator = new PlacesMenu();
 
@@ -148,6 +155,7 @@ function enable() {
     Main.panel.addToStatusArea('places-menu', _indicator, pos, 'left');
 }
 
+/** */
 function disable() {
     _indicator.destroy();
 }
