@@ -28,11 +28,15 @@ const MESSAGE_FADE_TIME = 2000;
 
 let text;
 
+/** */
 function hideMessage() {
     text.destroy();
     text = null;
 }
 
+/**
+ * @param {string} message - the message to flash
+ */
 function flashMessage(message) {
     if (!text) {
         text = new St.Label({ style_class: 'screenshot-sizer-message' });
@@ -67,6 +71,11 @@ let SIZES = [
     [720, 360], // Phone landscape fullscreen
 ];
 
+/**
+ * @param {Meta.Display} display - the display
+ * @param {Meta.Window=} window - for per-window bindings, the window
+ * @param {Meta.KeyBinding} binding - the key binding
+ */
 function cycleScreenshotSizes(display, window, binding) {
     // Probably this isn't useful with 5 sizes, but you can decrease instead
     // of increase by holding down shift.
@@ -82,7 +91,8 @@ function cycleScreenshotSizes(display, window, binding) {
 
     // Double both axes if on a hidpi display
     let scaleFactor = St.ThemeContext.get_for_stage(global.stage).scale_factor;
-    let scaledSizes = SIZES.map(size => size.map(wh => wh * scaleFactor));
+    let scaledSizes = SIZES.map(size => size.map(wh => wh * scaleFactor))
+        .filter(([w, h]) => w <= workArea.width && h <= workArea.height);
 
     // Find the nearest 16:9 size for the current window size
     let nearestIndex;
@@ -91,10 +101,6 @@ function cycleScreenshotSizes(display, window, binding) {
     for (let i = 0; i < scaledSizes.length; i++) {
         let [width, height] = scaledSizes[i];
 
-        // ignore sizes bigger than the workArea
-        if (width > workArea.width || height > workArea.height)
-            continue;
-
         // get the best initial window size
         let error = Math.abs(width - outerRect.width) + Math.abs(height - outerRect.height);
         if (nearestIndex === undefined || error < nearestError) {
@@ -105,10 +111,7 @@ function cycleScreenshotSizes(display, window, binding) {
 
     // get the next size up or down from ideal
     let newIndex = (nearestIndex + (backwards ? -1 : 1)) % scaledSizes.length;
-    let newWidth, newHeight;
-    [newWidth, newHeight] = scaledSizes[newIndex];
-    if (newWidth > workArea.width || newHeight > workArea.height)
-        [newWidth, newHeight] = scaledSizes[0];
+    let [newWidth, newHeight] = scaledSizes[newIndex];
 
     // Push the window onscreen if it would be resized offscreen
     let newX = outerRect.x;
@@ -118,8 +121,18 @@ function cycleScreenshotSizes(display, window, binding) {
     if (newY + newHeight > workArea.y + workArea.height)
         newY = Math.max(workArea.y + workArea.height - newHeight);
 
+    const id = window.connect('size-changed', () => {
+        window.disconnect(id);
+        _notifySizeChange(window);
+    });
     window.move_resize_frame(true, newX, newY, newWidth, newHeight);
+}
 
+/**
+ * @param {Meta.Window} window - the window whose size changed
+ */
+function _notifySizeChange(window) {
+    const { scaleFactor } = St.ThemeContext.get_for_stage(global.stage);
     let newOuterRect = window.get_frame_rect();
     let message = '%d×%d'.format(
         newOuterRect.width / scaleFactor,
@@ -135,6 +148,7 @@ function cycleScreenshotSizes(display, window, binding) {
     flashMessage(message);
 }
 
+/** */
 function enable() {
     Main.wm.addKeybinding(
         'cycle-screenshot-sizes',
@@ -150,6 +164,7 @@ function enable() {
         cycleScreenshotSizes);
 }
 
+/** */
 function disable() {
     Main.wm.removeKeybinding('cycle-screenshot-sizes');
     Main.wm.removeKeybinding('cycle-screenshot-sizes-backward');
