@@ -106,17 +106,20 @@ class WindowContextMenu extends PopupMenu.PopupMenu {
     }
 }
 
-const WindowTitle = GObject.registerClass(
 class WindowTitle extends St.BoxLayout {
-    _init(metaWindow) {
-        this._metaWindow = metaWindow;
+    static {
+        GObject.registerClass(this);
+    }
 
-        super._init({
+    constructor(metaWindow) {
+        super({
             style_class: 'window-button-box',
             x_expand: true,
             y_expand: true,
         });
 
+        this._metaWindow = metaWindow;
+
         this._icon = new St.Bin({ style_class: 'window-button-icon' });
         this.add(this._icon);
         this.label_actor = new St.Label({ y_align: Clutter.ActorAlign.CENTER });
@@ -175,30 +178,33 @@ class WindowTitle extends St.BoxLayout {
         this._metaWindow.disconnect(this._notifyWmClass);
         this._metaWindow.disconnect(this._notifyAppId);
     }
-});
+}
 
+class BaseButton extends St.Button {
+    static {
+        GObject.registerClass({
+            GTypeFlags: GObject.TypeFlags.ABSTRACT,
+            Properties: {
+                'ignore-workspace': GObject.ParamSpec.boolean(
+                    'ignore-workspace', 'ignore-workspace', 'ignore-workspace',
+                    GObject.ParamFlags.READWRITE,
+                    false),
+            },
+        }, this);
+    }
 
-const BaseButton = GObject.registerClass({
-    GTypeFlags: GObject.TypeFlags.ABSTRACT,
-    Properties: {
-        'ignore-workspace': GObject.ParamSpec.boolean(
-            'ignore-workspace', 'ignore-workspace', 'ignore-workspace',
-            GObject.ParamFlags.READWRITE,
-            false),
-    },
-}, class BaseButton extends St.Button {
-    _init(perMonitor, monitorIndex) {
-        this._perMonitor = perMonitor;
-        this._monitorIndex = monitorIndex;
-        this._ignoreWorkspace = false;
-
-        super._init({
+    constructor(perMonitor, monitorIndex) {
+        super({
             style_class: 'window-button',
             can_focus: true,
             x_expand: true,
             button_mask: St.ButtonMask.ONE | St.ButtonMask.THREE,
         });
 
+        this._perMonitor = perMonitor;
+        this._monitorIndex = monitorIndex;
+        this._ignoreWorkspace = false;
+
         this.connect('notify::allocation',
             this._updateIconGeometry.bind(this));
         this.connect('clicked', this._onClicked.bind(this));
@@ -240,6 +246,48 @@ const BaseButton = GObject.registerClass({
         this._updateVisibility();
     }
 
+    _setLongPressTimeout() {
+        if (this._longPressTimeoutId)
+            return;
+
+        const { longPressDuration } = Clutter.Settings.get_default();
+        this._longPressTimeoutId =
+            GLib.timeout_add(GLib.PRIORITY_DEFAULT, longPressDuration, () => {
+                delete this._longPressTimeoutId;
+
+                if (this._canOpenPopupMenu() && !this._contextMenu.isOpen)
+                    this._openMenu(this._contextMenu);
+                return GLib.SOURCE_REMOVE;
+            });
+    }
+
+    _removeLongPressTimeout() {
+        if (!this._longPressTimeoutId)
+            return;
+        GLib.source_remove(this._longPressTimeoutId);
+        delete this._longPressTimeoutId;
+    }
+
+    vfunc_button_press_event(buttonEvent) {
+        if (buttonEvent.button === 1)
+            this._setLongPressTimeout();
+        return super.vfunc_button_press_event(buttonEvent);
+    }
+
+    vfunc_button_release_event(buttonEvent) {
+        this._removeLongPressTimeout();
+
+        return super.vfunc_button_release_event(buttonEvent);
+    }
+
+    vfunc_touch_event(touchEvent) {
+        if (touchEvent.type === Clutter.EventType.TOUCH_BEGIN)
+            this._setLongPressTimeout();
+        else if (touchEvent.type === Clutter.EventType.TOUCH_END)
+            this._removeLongPressTimeout();
+        return super.vfunc_touch_event(touchEvent);
+    }
+
     activate() {
         if (this.active)
             return;
@@ -344,13 +392,15 @@ const BaseButton = GObject.registerClass({
             global.display.disconnect(this._windowLeftMonitorId);
         this._windowLeftMonitorId = 0;
     }
-});
+}
 
-
-const WindowButton = GObject.registerClass(
 class WindowButton extends BaseButton {
-    _init(metaWindow, perMonitor, monitorIndex) {
-        super._init(perMonitor, monitorIndex);
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor(metaWindow, perMonitor, monitorIndex) {
+        super(perMonitor, monitorIndex);
 
         this.metaWindow = metaWindow;
         this._skipTaskbarId = metaWindow.connect('notify::skip-taskbar', () => {
@@ -383,7 +433,7 @@ class WindowButton extends BaseButton {
             return;
         }
 
-        if (button === 1)
+        if (!button || button === 1)
             this._minimizeOrActivateWindow(this.metaWindow);
         else
             this._openMenu(this._contextMenu);
@@ -422,8 +472,7 @@ class WindowButton extends BaseButton {
         global.display.disconnect(this._notifyFocusId);
         this._contextMenu.destroy();
     }
-});
-
+}
 
 class AppContextMenu extends PopupMenu.PopupMenu {
     constructor(source) {
@@ -483,10 +532,13 @@ class AppContextMenu extends PopupMenu.PopupMenu {
     }
 }
 
-const AppButton = GObject.registerClass(
 class AppButton extends BaseButton {
-    _init(app, perMonitor, monitorIndex) {
-        super._init(perMonitor, monitorIndex);
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor(app, perMonitor, monitorIndex) {
+        super(perMonitor, monitorIndex);
 
         this.app = app;
         this._updateVisibility();
@@ -627,7 +679,7 @@ class AppButton extends BaseButton {
         if (contextMenuWasOpen)
             this._contextMenu.close();
 
-        if (button === 1) {
+        if (!button || button === 1) {
             if (menuWasOpen)
                 return;
 
@@ -670,16 +722,15 @@ class AppButton extends BaseButton {
         this.app.disconnect(this._windowsChangedId);
         this._menu.destroy();
     }
-});
+}
 
-
-const WindowList = GObject.registerClass(
 class WindowList extends St.Widget {
-    _init(perMonitor, monitor) {
-        this._perMonitor = perMonitor;
-        this._monitor = monitor;
+    static {
+        GObject.registerClass(this);
+    }
 
-        super._init({
+    constructor(perMonitor, monitor) {
+        super({
             name: 'panel',
             style_class: 'bottom-panel solid',
             reactive: true,
@@ -688,6 +739,9 @@ class WindowList extends St.Widget {
         });
         this.connect('destroy', this._onDestroy.bind(this));
 
+        this._perMonitor = perMonitor;
+        this._monitor = monitor;
+
         let box = new St.BoxLayout({ x_expand: true, y_expand: true });
         this.add_actor(box);
 
@@ -765,10 +819,9 @@ class WindowList extends St.Widget {
 
         let workspaceManager = global.workspace_manager;
 
-        this._workspaceSignals = new Map();
         this._nWorkspacesChangedId = workspaceManager.connect(
-            'notify::n-workspaces', this._onWorkspacesChanged.bind(this));
-        this._onWorkspacesChanged();
+            'notify::n-workspaces', this._updateWorkspaceIndicatorVisibility.bind(this));
+        this._updateWorkspaceIndicatorVisibility();
 
         this._switchWorkspaceId = global.window_manager.connect(
             'switch-workspace', this._checkGrouping.bind(this));
@@ -779,7 +832,7 @@ class WindowList extends St.Widget {
         });
 
         this._overviewHidingId = Main.overview.connect('hidden', () => {
-            this.visible = !Main.layoutManager.primaryMonitor.inFullscreen;
+            this.visible = !this._monitor.inFullscreen;
             this._updateKeyboardAnchor();
         });
 
@@ -791,6 +844,10 @@ class WindowList extends St.Widget {
                 this._updateKeyboardAnchor();
             });
 
+        this._windowSignals = new Map();
+        this._windowCreatedId = global.display.connect(
+            'window-created', (dsp, win) => this._addWindow(win));
+
         this._dragBeginId = Main.xdndHandler.connect('drag-begin',
             this._monitorDrag.bind(this));
         this._dragEndId = Main.xdndHandler.connect('drag-end',
@@ -913,7 +970,7 @@ class WindowList extends St.Widget {
                        w2.metaWindow.get_stable_sequence();
             });
             for (let i = 0; i < windows.length; i++)
-                this._onWindowAdded(null, windows[i].metaWindow);
+                this._addWindow(windows[i].metaWindow);
         } else {
             let apps = this._appSystem.get_running().sort((a1, a2) => {
                 return _getAppStableSequence(a1) -
@@ -953,7 +1010,7 @@ class WindowList extends St.Widget {
             child.destroy();
     }
 
-    _onWindowAdded(ws, win) {
+    _addWindow(win) {
         if (!this._grouped)
             this._checkGrouping();
 
@@ -964,21 +1021,26 @@ class WindowList extends St.Widget {
         if (children.find(c => c.metaWindow === win))
             return;
 
+        this._windowSignals.set(
+            win, win.connect('unmanaged', () => this._removeWindow(win)));
+
         let button = new WindowButton(win, this._perMonitor, this._monitor.index);
         this._settings.bind('display-all-workspaces',
             button, 'ignore-workspace', Gio.SettingsBindFlags.GET);
         this._windowList.add_child(button);
     }
 
-    _onWindowRemoved(ws, win) {
+    _removeWindow(win) {
         if (this._grouped)
             this._checkGrouping();
 
         if (this._grouped)
             return;
 
-        if (win.get_compositor_private())
-            return; // not actually removed, just moved to another workspace
+        const id = this._windowSignals.get(win);
+        if (id)
+            win.disconnect(id);
+        this._windowSignals.delete(win);
 
         let children = this._windowList.get_children();
         let child = children.find(c => c.metaWindow === win);
@@ -986,39 +1048,6 @@ class WindowList extends St.Widget {
             child.destroy();
     }
 
-    _onWorkspacesChanged() {
-        let workspaceManager = global.workspace_manager;
-        let numWorkspaces = workspaceManager.n_workspaces;
-
-        for (let i = 0; i < numWorkspaces; i++) {
-            let workspace = workspaceManager.get_workspace_by_index(i);
-            if (this._workspaceSignals.has(workspace))
-                continue;
-
-            let signals = { windowAddedId: 0, windowRemovedId: 0 };
-            signals._windowAddedId = workspace.connect_after(
-                'window-added', this._onWindowAdded.bind(this));
-            signals._windowRemovedId = workspace.connect(
-                'window-removed', this._onWindowRemoved.bind(this));
-            this._workspaceSignals.set(workspace, signals);
-        }
-
-        this._updateWorkspaceIndicatorVisibility();
-    }
-
-    _disconnectWorkspaceSignals() {
-        let workspaceManager = global.workspace_manager;
-        let numWorkspaces = workspaceManager.n_workspaces;
-
-        for (let i = 0; i < numWorkspaces; i++) {
-            let workspace = workspaceManager.get_workspace_by_index(i);
-            let signals = this._workspaceSignals.get(workspace);
-            this._workspaceSignals.delete(workspace);
-            workspace.disconnect(signals._windowAddedId);
-            workspace.disconnect(signals._windowRemovedId);
-        }
-    }
-
     _monitorDrag() {
         DND.addDragMonitor(this._dragMonitor);
     }
@@ -1082,18 +1111,20 @@ class WindowList extends St.Widget {
         Main.keyboard._bottomDragAction.disconnect(this._keyboardVisiblechangedId);
         this._keyboardVisiblechangedId = 0;
 
-        this._disconnectWorkspaceSignals();
         global.workspace_manager.disconnect(this._nWorkspacesChangedId);
         this._nWorkspacesChangedId = 0;
 
         global.window_manager.disconnect(this._switchWorkspaceId);
         this._switchWorkspaceId = 0;
 
+        this._windowSignals.forEach((id, win) => win.disconnect(id));
+        this._windowSignals.clear();
 
         Main.overview.disconnect(this._overviewShowingId);
         Main.overview.disconnect(this._overviewHidingId);
 
         global.display.disconnect(this._fullscreenChangedId);
+        global.display.disconnect(this._windowCreatedId);
 
         this._stopMonitoringDrag();
         Main.xdndHandler.disconnect(this._dragBeginId);
@@ -1105,7 +1136,7 @@ class WindowList extends St.Widget {
         for (let i = 0; i < windows.length; i++)
             windows[i].metaWindow.set_icon_geometry(null);
     }
-});
+}
 
 class Extension {
     constructor() {
